
String.prototype.splice = function(
    index,
    howManyToDelete,
    stringToInsert /* [, ... N-1, N] */
){
    // Create a character array out of the current string
    // by splitting it. In the context of this prototype
    // method, THIS refers to the current string value
    // being spliced.
    var characterArray = this.split( "" );
    // Now, let's splice the given strings (stringToInsert)
    // into this character array. It won't matter that we
    // are mix-n-matching character data and string data as
    // it will utlimately be joined back into one value.
    //
    // NOTE: Because splice() mutates the actual array (and
    // returns the removed values), we need to apply it to
    // an existing array to which we have an existing
    // reference.
    Array.prototype.splice.apply(
        characterArray,
        arguments
    );
    // To return the new string, join the character array
    // back into a single string value.
    return(
        characterArray.join( "" )
    );
};

prism.on("beforemenu", function(e, args){

    // apply only for menu inside widget editor
    if (defined(args.ui.css) && args.ui.css === "w-menu-host") {
        // adding the option to the menu
        args.settings.items.push(
            {
                type: "separator"
            },
            {
                "caption": "Sync dashboard filters with measures",
                "execute": syncFiltersWithMeasures
            }
        );
    }

    if ($$get(args,"settings.name") === "widget-metadataitem" && !(args.ui.target.hasClass('f-header-host'))) {
        args.settings.items.push(
            {
                type: "separator"
            },
            {
                "caption" : "Sync Measure with filters",
                "execute" : syncSingleMeasure,
                "item" : args.settings.item
            }
        );
    }
});

    // sync method for syncing all filters with parameters syntax (@)
var syncFiltersWithMeasures = function(){
        var filters = $.extend(true,[],prism.activeDashboard.filters.flatten()),
            disabled = prism.activeWidget.metadata.ignore.dimensions,
            measures=[], item, jaql, filter, guid, i, j, k, m;

        if (prism.activeWidget.type === 'indicator'){
            measures[parameterConfig.indicatorPrimaryIndex] = prism.activeWidget.metadata.panels[parameterConfig.indicatorPrimaryIndex];
            measures[parameterConfig.indicatorSecondaryIndex] = prism.activeWidget.metadata.panels[parameterConfig.indicatorSecondaryIndex];
        } else{
            measures[parameterConfig.measuresIndex] = prism.activeWidget.metadata.panels[parameterConfig.measuresIndex];
        }

        // if there are disabled filters inside the widget, we should ignore them and consider them as out of filters list
        ignoreDisabledFilters(filters,disabled);

        // add filters to measures
        for (i = 0 ; i < filters.length ; i++){
            filter = filters[i];
            for (k = 0 ; k < measures.length ; k++){
                m = measures[k];
                if (m){
                    for (j = 0 ; j < m.items.length ; j++){
                        // get the item for later use in order to update field property
                        item = m.items[j];
                        jaql = item.jaql;

                        // checks if jaql contains formula. if does, drill each context and add filter
                        if (jaql.formula && jaql.context){
                            addFilterToContextRec(jaql, filter,item);
                        }
                        // case of jaql is general dimension - change it to formula and add the filter to the formula/context
                        else {
                            //create context prop'
                            jaql.context = {};
                            // create new guid
                            guid = "[" + $$guid_fast(9) + "]";
                            // copy the jaql into the new context[guid]
                            jaql.context[guid] = $.extend(true,{},jaql);
                            // create formula
                            jaql.formula = "(" + jaql.agg + "(" + guid + "))";
                            // delete context remains
                            delete jaql.context[guid].context;
                            // after creating formula/context - need to delete all regular properties from the object
                            deleteProperties(jaql,["agg","table","dim","datatype","column"]);
                            // delete the agg property to avoid duplicate aggs (inside formula and inside actual dimension)
                            delete jaql.context[guid].agg;
                            // adding type of measure
                            jaql.type = "measure";

                            // adding the filter to the context
                            addFilterToConetxt(jaql,filter,item);
                        }
                    }
                }
            }
        }

        for (k = 0 ; k < measures.length ; k++){
            m = measures[k];

            if (m){
                // remove filters from measures
                for (i = 0 ; i < m.items.length ; i++){
                    item = m.items[i];
                    jaql = item.jaql;

                    // if formula exists - check and remove filters from jaql.
                    if (jaql.formula && jaql.context)
                    {
                        removeFiltersFromContextRec(jaql,filters,item);
                    }
                }
            }
        }

        // refresh the widget in order to update data with new metadata
        prism.activeWidget.refresh();
    },

    // sync method for syncing a single item (measure) with all filters with parameters syntax (@)
    syncSingleMeasure = function(){
        var filters = $.extend(true,[],prism.activeDashboard.filters.flatten()),
            disabled = prism.activeWidget.metadata.ignore.dimensions,
            item = this.item,
            filter, jaql, guid;

        // if there are disabled filters inside the widget, we should ignore them and consider them as out of filters list
        ignoreDisabledFilters(filters,disabled);

        for (var i = 0 ; i < filters.length ; i++){
            filter = filters[i];
            if (item){
                jaql = item.jaql;

                // checks if jaql contains formula. if does, drill each context and add filter
                if (jaql.formula && jaql.context) {
                    addFilterToContextRec(jaql, filter,item);
                }
                // case of jaql is general dimension - change it to formula and add the filter to the formula/context
                else {
                    //create context prop'
                    jaql.context = {};
                    // create new guid
                    guid = "[" + $$guid_fast(9) + "]";
                    // copy the jaql into the new context[guid]
                    jaql.context[guid] = $.extend(true,{},jaql);
                    // create formula
                    jaql.formula = "(" + jaql.agg + "(" + guid + "))";
                    // delete context remains
                    delete jaql.context[guid].context;
                    // after creating formula/context - need to delete all regular properties from the object
                    deleteProperties(jaql,["agg","table","dim","datatype","column"]);
                    // delete the agg property to avoid duplicate aggs (inside formula and inside actual dimension)
                    delete jaql.context[guid].agg;
                    // adding type of measure
                    jaql.type = "measure";

                    // adding the filter to the context
                    addFilterToConetxt(jaql,filter,item);
                }
            }
        }

        if (item){
            jaql = item.jaql;

            // if formula exists - check and remove filters from jaql.
            if (jaql.formula && jaql.context) {
                // remove filters from measure
                removeFiltersFromContextRec(jaql,filters,item);
            }
        }

        // refresh the widget in order to update data with new metadata
        prism.activeWidget.refresh();
    },

    // get object and array of string values and if value defined in the object - it will be deleted
    deleteProperties = function(obj,propsArray) {

    for (var key in propsArray){
        if (obj.hasOwnProperty(propsArray[key])){
            delete obj[propsArray[key]];
        }
    }
},

    // Recursive function to drill inside nested contexts and check if filter exists - If it doesn't, add it.
    addFilterToContextRec = function(jaql, filter,item){

        var isExist = false;
        for (var k in jaql.context){
            if (!jaql.context.hasOwnProperty(k)){
                continue;
            }

            // if there is a nested context (like saved formula), drill
            if (jaql.context[k].context){
                addFilterToContextRec(jaql.context[k],filter,item);
                // setting to true in order to avoid of getting the addFilterToConetxt() when drilling
                isExist = true;
            }
            // case there is no nested context, checks if filter exists
            else if (jaql.context[k].dim === filter.jaql.dim){
                // is it date type?
                if (filter.jaql.level){
                    // If does, the levels are equal?
                    if (filter.jaql.level === jaql.context[k].level){
                        // if exists, is it already start with @?
                        if (!jaql.context[k].title.startsWith('@'))
                        {
                            jaql.context[k].title = '@'.concat(jaql.context[k].title);
                        }

                        compareTitles(jaql,k,filter);
                        isExist = true;
                    }
                } else {
                    // if exists, is it already start with @?
                    if (!jaql.context[k].title.startsWith('@')) {
                        jaql.context[k].title = '@'.concat(jaql.context[k].title);
                    }

                    compareTitles(jaql,k,filter);
                    isExist = true;
                }
            }
        }

        // if filter doesn't exist, add it
        if (!isExist) {
            // the actual adding function
            addFilterToConetxt(jaql,filter,item);
        }
    },

    // The actual adding filter to context function
    addFilterToConetxt = function(jaql,filter,item){
        var extendedFilter,
            guid;

        // copy filter with $.extend to avoid reference issues (editing the object)
        extendedFilter = $.extend(true,{}, filter.jaql);
        // creates new guid for context
        guid = "[" + $$guid_fast(9) + "]";

        // add the filter to the context
        jaql.context[guid] = extendedFilter;
        // delete the actual filter prop' from copied filter
        // (we need just to add it as a dimension filter - logics for actual filter happens in parameters.js)
        delete jaql.context[guid].filter;
        // adding @ to begining of title for logic to work inside parameters.js
        jaql.context[guid].title = '@'.concat(jaql.context[guid].title);
        // update formula with new filter guid
        if (/^\(.*\)$/.test(jaql.formula)) {
            jaql.formula = jaql.formula.splice(jaql.formula.lastIndexOf(')'),0,',' + guid);
        } else {
            jaql.formula = '(' + jaql.formula + ')';
            jaql.formula = jaql.formula.splice(jaql.formula.lastIndexOf(')'),0,',' + guid);
        }

        // if there is a field prop inside item - update its id with new formula
        if (defined(item.field)) {
            item.field.id = jaql.formula;
        }
    },

    // a function to remove turned off filters from the measures
    removeFiltersFromContextRec = function(jaql,filters,item){

        for (var k in jaql.context) {
            var isExist = false;
            if (!jaql.context.hasOwnProperty(k)) {
                continue;
            }

            // if there is a nested context (like saved formula), drill
            if (jaql.context[k].context) {
                removeFiltersFromContextRec(jaql.context[k],filters,item);
                // setting to true in order to avoid of deleting context that has nested context
                isExist = true;
            } else {
                // if there is agg property - means it's not a filter so we skip it
                // also, assuming dimension is in first position inside the formula
                if (jaql.context[k].agg || jaql.formula.split(',')[0].indexOf(k) > -1) {
                    continue;
                }

                // going through each filter and checking if filter from context exists in filters list
                for (var i = 0 ; i < filters.length ; i++) {
                    if (jaql.context[k].dim === filters[i].jaql.dim) {
                        if (filters[i].jaql.level) {
                            if (filters[i].jaql.level === jaql.context[k].level) {
                                isExist = true;
                                // if exist, no need to continue with loop
                                break;
                            }
                        } else {
                            isExist = true;
                            // if exist, no need to continue with loop
                            break;
                        }
                    }
                }

                // if context has filter that isn't a part of dashboard filters list - delete it
                if (!isExist) {
                    // delete the entire filter object from context
                    delete jaql.context[k];
                    // delete the guid filter from the formula
                    jaql.formula = jaql.formula.replace(','+k,"").replace(', '+k,"");

                    // if there is a field prop inside item - update its id with new formula
                    if (defined(item.field)) {
                        item.field.id = jaql.formula;
                    }
                }
            }
        }
    },

    // a function to remove disabled filters from filters list (ignore filters)
    ignoreDisabledFilters = function(filters,disabled){
        if (disabled.length > 0) {
            for (var index = 0 ; index < filters.length ; index++) {
                for (var index2 = 0 ; index2 < disabled.length ; index2++) {
                    if (disabled[index2] === filters[index].jaql.dim) {
                        // delete filter that in ignore list
                        filters.splice(index,1);
                    }
                }
            }
        }
    },

    // update item title with filter title if there is any difference
    compareTitles = function(itemJaql,contextID,filter){
        var title,tagBefore,
            itemTitle = itemJaql.context[contextID].title,
            filterTitle = filter.jaql.title;
        if (itemTitle.startsWith('@.PREV.')) {
            var prev = itemTitle.match(/^@\.PREV\.(\d)\./);
            if (prev) {
                tagBefore = prev[0];
                title = itemTitle.substring(9);
            }
            else {
                tagBefore = "@.PREV.";
                title = itemTitle.substring(7);
            }
        } else if (itemTitle.startsWith('@')) {
            tagBefore = "@";
            title = itemTitle.substring(1);
        }

        if (title !== filterTitle && tagBefore) {
            itemJaql.context[contextID].title = tagBefore + filterTitle;
        }
    };