//
// parameters plugin.
//

prism.run(["$jaql", function ($jaql) {
    //
    // Query Handling
    // --------------

    // drill through the given object and look for available parameters in the context matching any of the object's placeholders
    function drill(o, context, widget) {

        for (var p in o) {
            var propValue, m, a, prev;
            var newFilter = {};

            if (!o.hasOwnProperty(p)) {
                continue;
            }

            propValue = o[p];
            var t = $$get(propValue, "title");

            // looking for placeholder @.PREV. - data should filter with previous year.
            handlePrevParameter(o, p, t, context);
            // looking for placeholder @ - data should filter with filtered date.
            handleParameter(o, p, t, context);
            // looking for drill hierarchy dimension to add a filter if has one
            //handleDrilledFilter(o, p, context, widget);

            // continue drilling
            if (Array.isArray(propValue) || typeof (propValue) === "object") {
                drill(propValue, context, widget);
            }
        }
    }

    // looking for placeholder '@.PREV.' on measure title and adds the filter from dashboard minus 1 year (or more) to the measure and remove from scope (valid only with date dimension)
    function handlePrevParameter(o, p, t, context) {

        var m, a, prev;
        var newFilter = {};

        if (typeof t === 'string' && t.startsWith('@.PREV.')) {
            prev = t.match(/^@\.PREV\.(\d)\./);
            if (prev) {
                prev = parseInt(prev[1]);
                t = t.substring(9);
            }
            else {
                prev = 1;
                t = t.substring(7);
            }

            a = m = context[t];

            if (defined(m)) {

                if (m.title !== context.$metadata[0].jaql.title) {
                    // clone filter to make changes that will not affect real filter
                    newFilter = $.extend(true, newFilter, $$.object.attrs(m,['column','filter','datatype','table','title','dim','level']));
                    handleBackgroundFilter(newFilter);

                    if (!m.filter.all) {
                        // case user is using calendar option
                        if (newFilter.filter.from && newFilter.filter.to) {
                            modifyCalendarFilter(newFilter, prev);
                            o[p] = newFilter;
                            o[p].modifiedByParameters = true;
                            o[p].prevValue = prev;
                        }
                        // case user is using regular option
                        else {
                            modifyDateFilter(newFilter, prev);
                            // applying value
                            o[p] = newFilter;
                            o[p].modifiedByParameters = true;
                            o[p].prevValue = prev;
                        }
                        // remove custom filter from context
                        if (defined(a, "$$guid")) {

                            sisenseUtils.array.remove(context.$metadata, context.$filtersmap[a.$$guid]);
                        }
                    }
                    else {
                        o[p] = newFilter;
                        o[p].modifiedByParameters = true;

                        // remove custom filter from context
                        if (defined(a, "$$guid")) {

                            sisenseUtils.array.remove(context.$metadata, context.$filtersmap[a.$$guid]);
                        }
                    }
                }
            }
        }
    }

    // looking for placeholder '@' on measure title and adds the filter from dashboard to the measure and remove from scope
    function handleParameter(o, p, t, context) {

        var m, a;
        var newFilter = {};


        if (typeof t === 'string' && t.startsWith('@')) {

            t = t.substring(1);
            a = m = context[t];

            if (!defined(m) && /\.value$/.test(t)/* t ends with '.value' */) {

                t = t.substring(0, t.indexOf(".value"));
                a = context[t];
                m = parseFloat($$get(a, "filter.equals"));
            }

            // custom value provider
            if (defined(context.$custom)) {

                m = null;
                a = context.$custom(t) || a;
            }


            if (defined(m)) {

                if (m.title !== context.$metadata[0].jaql.title) {
                    // clone filter to make changes that will not affect real filter
                    newFilter = $.extend(true, newFilter, $$.object.attrs(m,['column','filter','datatype','table','title','dim','level']));

                    handleBackgroundFilter(newFilter);
                    // applying value
                    o[p] = newFilter;
                    o[p].modifiedByParameters = true;
                }


                // remove custom filter from context
                if (defined(a, "$$guid")) {
                    sisenseUtils.array.remove(context.$metadata, context.$filtersmap[a.$$guid]);

                }
            }
        }
    }


    function isBackgroundFilter(item) {
        return (defined(item) && defined(item, 'filter.filter'));
    }

    function hasExclude(item) {
        return (defined(item, 'filter.exclude') && angular.isArray(item.filter.exclude.members));
    }

    function hasInclude(item) {
        return (defined(item, 'filter.members') && angular.isArray(item.filter.members));
    }

    function handleBackgroundFilter(item) {

        var members,
            backgroundFilterHasExclude,
            backgroundFilterHasInclude,
            filterHasExclude,
            filterHasInclude;

        if (!isBackgroundFilter(item)) {

            return;
        }

        backgroundFilterHasExclude = hasExclude(item.filter);
        backgroundFilterHasInclude = hasInclude(item.filter);
        filterHasExclude = hasExclude(item);
        filterHasInclude = hasInclude(item);

        // real filter + background filter, so exclude + include means
        // in the real filter there are members included and in the background filter
        // there are members excluded
        if (filterHasExclude) {
            //exclude + include
            if (backgroundFilterHasInclude) { // check bg filter

                // return difference between arrays
                members = item.filter.filter.members.filter(function (i) {
                    return item.filter.exclude.members.indexOf(i) < 0;
                });
                item.filter.members = members;
            }
            //exclude + exclude
            else if (backgroundFilterHasExclude) {	// check bg filter

                members = item.filter.filter.exclude.members.concat(item.filter.exclude.members);

                item.filter.exclude.members = members;
            }
        }
        else if (filterHasInclude) {
            //include + include
            if (backgroundFilterHasInclude) {
                //special case of unselecting all
                if (item.filter.members.length === 0) {
                    members = item.filter.filter.members;
                    item.filter.members = members;
                }

            }
            //include + exclude
            //no need to do anything
        }
        else if (item.filter.all === true) {
            item.filter = item.filter.filter;
        }
        item.filter.filter = undefined;

        return item;

    }

    // looking for drill hierarchy dimension to add it a filter from the dashboard if exist
    function handleDrilledFilter(o, p, context, widget) {

        var t, m, items, widgetMetadataThroughItem = null;

        if (defined(o[p], "through")) {
            t = o[p].through.jaql.title;

            if (typeof t === 'string') {
                m = context[t];
                if (m) {
                    o[p].through.jaql = m;

                    items = widget.metadata.items();

                    for (var i = 0; i < items.length && !widgetMetadataThroughItem; i++) {
                        widgetMetadataThroughItem = getMetadataThroughItem(items[i], t);
                    }

                    if (defined(widgetMetadataThroughItem)) {

                        widgetMetadataThroughItem.jaql = m;
                        widget.changesMade();
                    }
                }
            }
        }
    }

    function getMetadataThroughItem(item, title) {

        var prop;

        prop = item["through"];
        if (prop) {
            if (defined(prop, "jaql") && prop.jaql.title === title) {
                return prop;
            }
            else {
                if (prop = item["parent"]) {
                    return getMetadataThroughItem(prop, title);
                }
            }
        }
        else {
            return null;
        }
    }

    // reduce year in calendar filter according to prev value
    function modifyCalendarFilter(filter, prev) {

        var dateTo = moment(filter.filter.to).toDate(),
            dateFrom = moment(filter.filter.from).toDate();

        // create new range with last year
        filter.filter.from = new Date(dateFrom.getFullYear() - prev, dateFrom.getMonth(), dateFrom.getDate()).toISOString().substring(0, dateFrom.toISOString().indexOf('T'));
        filter.filter.to = new Date(dateTo.getFullYear() - prev, dateTo.getMonth(), dateTo.getDate()).toISOString().substring(0, dateTo.toISOString().indexOf('T'));

    }

    // reduce year in date filter according to prev value
    function modifyDateFilter(filter, prev) {

        for (var i = 0; i < filter.filter.members.length; i++) {
            // create new members for each one with year before.
            //var date = window.sisenseUtils.date.parseDate(filter.filter.members[i]);
            //moment;
            //date.setMonth(date.getMonth() - 3);
            // var date = moment(filter.filter.members[i]).subtract(3, 'months').format('YYYY-MM-DD');
            var date = moment(filter.filter.members[i]).subtract(prev, 'years').format('YYYY-MM-DD');
            filter.filter.members[i] = date;
            //filter.filter.members[i] = date.toISOString().substring(0, date.toISOString().indexOf('T'));
        }
    }

    //
    // generate name -> value context map that will be used to apply parameter values to query placeholders.
    // Also, a custom value provider can be used when attached to the widget.options object to allow resolving placeholders values in ways exstinging the default widget/dashboard filters.
    //
    // I/E -
    //
    // dashboard.$customParameterValueProvider = function(name) {
    //
    //		if (name === "A") return { ... };
    // }
    function buildContext(context, arr) {

        var f, t;

        for (var i = 0; i < arr.length; i++) {

            f = arr[i];
            t = $$get(f, "jaql.title");

            if (defined(t)) {

                context[t] = f.jaql;

                //
                // mapping source -> query item to remove filters that were used as parameter value
                var mitem = context.$metadata.filter(function (m) {
                    if (f.jaql.customPeriod) {
                        return m.panel === "scope" && !$jaql.isMeasure(m) && $$get(m, "jaql.dim") === $$get(f, "jaql.dim") && $$get(m, "jaql.customPeriod.filter.level") === $$get(f, "jaql.customPeriod.filter.level");
                    }
                    return m.panel === "scope" && !$jaql.isMeasure(m) && $$get(m, "jaql.dim") === $$get(f, "jaql.dim") && $$get(m, "jaql.level") === $$get(f, "jaql.level");


                });
                if (mitem.length === 1) {

                    f.jaql.$$guid = $$guid(8);
                    context.$filtersmap[f.jaql.$$guid] = mitem[0];
                }

            }
        }
    }


    function getContext(widget, query) {

        var context = {};
        context.$filtersmap = {};
        context.$metadata = query.metadata;

        buildContext(context, widget.dashboard.filters.flatten());
        buildContext(context, widget.metadata.filters());

        // setting custom provider
        context.$custom = $$get(widget.dashboard, "$customParameterValueProvider");

        return context;
    }

    // gets the widget before query event and tryng to apply parameter values to existing query placeholders
    function processWidgetQuery(s, e) {

        var widget = e.widget;
        var query = e.query;

        // generating context
        var context = getContext(widget, query);

        //TEMP!!!
        //var filters = [];
        //if(widget.type === "pivot")
        //{
        //	filters = e.query.metadata.filter(function(item){return item.panel=== "scope"});
        //}

        // applying parameter values;
        drill(query.metadata, context, widget);

        //TEMP!!!
        // drill function removes some of the filters, we need to get them back to query metadata
        //filters.forEach(function(filterItem){
        //	//get remaning filters agter drill function
        //	var existingFilter = e.query.metadata.filter(function(item){
        //		return item.panel === "scope" && item.jaql.dim === filterItem.jaql.dim;
        //	});
        //
        //	// if no filter with same dimension exists then we should add it back to query metadata
        //	if(!existingFilter.length){
        //		e.query.metadata = e.query.metadata.concat(filterItem);
        //	}
        //})


        // cleaning $$guid attributes from context items
        cleanGuid(context);
    }

    function cleanGuid(context) {

        for (var p in context) {

            if (!context.hasOwnProperty(p)) {

                continue;
            }

            if (defined(context, p + ".$$guid")) {

                delete context[p].$$guid;
            }
        }
    }

    // function to check if PeriodChanger plugin is exist and enabled
    function checkPeriodPlugin() {
        var hasPeriodChanger = false,
            widgets = prism.activeDashboard.widgets.$$widgets;

        for (var i = 0; i < widgets.length && !hasPeriodChanger; i++) {
            if (widgets[i].type === "periodChanger" && !widgets[i].disabled)
                hasPeriodChanger = true;
        }
        return hasPeriodChanger;
    }


    //
    // Hooking-up
    // ----------

    var handler;

    // uses to intercept all widget queries of a given dashboard and apply dynamic values into predefined placeholders
    function dashboardHandler(dashboard) {

        // release all references held by the dashboard handler
        this.destory = function () {

            dashboard.off("widgetbeforequery", processWidgetQuery);
        };

        // hooking up to the dashboard
        dashboard.on("widgetbeforequery", processWidgetQuery);
    }

    // hooking up to all dashboard queries on load
    prism.on("dashboardloaded", function (e, args) {

        handler = new dashboardHandler(args.dashboard);
    });

    // removing hooks from dashboard on unload
    prism.on("dashboardunloaded", function () {

        if (!defined(handler)) {

            return;
        }

        handler.destory();
        handler = null;
    });
}]);


